/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.filestore;

import org.globus.tutorial.types.filestore.FileStoreException;

import org.globus.wsrf.utils.FaultHelper;

/**
 * Class which handles constants associated with the filestore
 * and also exceptions.
 *
 */
public class Constants {

    /** The base path in the address for filestore services */
    public static final String SERVICE_BASE_PATH = "/wsrf/services/";

    /** The name of the service used in the service address */
    public static final String SERVICE_PATH = "FileService";

    /** The name of the factory service used in the factory service address */
    public static final String FACTORY_PATH = "FileStoreFactoryService";

    /** The name of the service used in the service address */
    public static final String PERSIST_SERVICE_PATH = 
        "PersistentFileService";

    /** The name of the factory service used in the factory service address */
    public static final String PERSIST_FACTORY_PATH = 
        "PersistentFileStoreFactoryService";

    /** The unique name for the filestore for defining properties of the
	filestore */
    public static final String NS = 
        "http://org.globus.tutorial/filestore";

    /** 
     * Contructs a FileStoreException using an
     * error message.
     * @param errMsg the error message describing the source of the error.
     * @return the filestore exception containing the error message.
     */
    public static FileStoreException makeFault(String errMsg) {
        return makeFault(errMsg, null);
    }

    /**
     * Constructs a FileStoreException from an error message and
     * the type of exception thrown.
     * @param errMsg the error message describing the source of the error.
     * @param exp the exception object which was thrown.
     * @return the filestore exception containing the error message.
     */
    public static FileStoreException makeFault(String errMsg, Exception exp) {
        FileStoreException fault = new FileStoreException();
        FaultHelper helper = new FaultHelper(fault);
        if (errMsg != null) {
            helper.setDescription(errMsg);
        }
        if (exp != null) {
            helper.addFaultCause(exp);
        }

        return (FileStoreException)helper.getFault();
    }
}

