/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.filestore.service;

import org.globus.wsrf.encoding.ObjectDeserializer;

import org.globus.tutorial.impl.filestore.Constants;

import javax.xml.namespace.QName;

import org.apache.axis.message.MessageElement;

import org.globus.tutorial.types.filestore.VoidType;
import org.globus.tutorial.types.filestore.VoidType1;
import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.FileStoreException;
import org.globus.tutorial.types.filestore.ArrayOfDouble;
import org.globus.tutorial.types.filestore.ConcatInput;
import org.globus.tutorial.types.filestore.RequestFileType;

import org.globus.wsrf.container.ServiceHost;

import org.globus.wsrf.utils.AddressingUtils;

import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceHome;
import org.globus.wsrf.ResourceContext;

import java.rmi.RemoteException;

import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.wsrf.impl.SimpleResourceKey;

import org.globus.tutorial.filestore.service.FileServiceAddressingLocator;

import org.oasis.wsrf.lifetime.WSResourceLifetimeServiceAddressingLocator;
import org.oasis.wsrf.lifetime.ImmediateResourceTermination;

import java.util.Vector;

import javax.xml.rpc.ServiceException;

import org.globus.wsrf.impl.SimpleResourceKey;

import javax.naming.InitialContext;

public class FileService {

    public VoidType store(Data parameters) 
        throws RemoteException, FileStoreException {
        FileServiceResource resource =
            (FileServiceResource)ResourceContext.getResourceContext()
            .getResource();
        resource.store(parameters);
        return new VoidType();
    }

    public Data fetch(VoidType parameters) 
        throws RemoteException, FileStoreException {
        FileServiceResource resource =
            (FileServiceResource)ResourceContext.getResourceContext()
            .getResource();
        return resource.fetch();
    }

    public VoidType eraseContents(VoidType1 parameters) 
        throws RemoteException, FileStoreException {

        FileServiceResource resource =
            (FileServiceResource)ResourceContext.getResourceContext()
            .getResource();
        resource.delete();
        return new VoidType();
    }

    public VoidType concatenate(ConcatInput parameters) 
        throws RemoteException, FileStoreException, ServiceException {

        EndpointReferenceType[] eprs = parameters.getInputFiles().getValue();
        EndpointReferenceType output = parameters.getOutputFile();
        
        if (eprs == null) {
	    throw Constants.makeFault("FileStore: Concatenation: "+
					 " input array is null");
        }
        
        //        if (eprs.length < 2) {
        //	    throw Constants.makeFault("FileStore: Concatenation: "+
        //					 "must have two files"+
        //					 " to cat.");
        //        }



        Data[] dataArray = null;
        int dataArrayIndex = 0;

        FileServiceResource outResource = null;
        try {
            outResource = (FileServiceResource)getResource(output);
        } catch (Exception exp) {
            exp.printStackTrace();
            throw new RemoteException(exp.getMessage());
        }
        Data outputData = outResource.fetch();
        if (outputData != null) {
            dataArray = new Data[eprs.length + 1];
            dataArray[0] = outputData;
            dataArrayIndex++;
            // really only erasing the data in there, since it is
            // delete directly on resource
            outResource.delete();
        } else {
            dataArray = new Data[eprs.length];
        }

        FileServiceResource resource = null;

	for(int i = 0; i < eprs.length; i++){
	    if(eprs[i]==null) {
		throw Constants.makeFault("Concatenation: an epr is null ");
            }
            try {
                resource = (FileServiceResource)getResource(eprs[i]);
            } catch (Exception exp) {
                throw new RemoteException(exp.getMessage());
            }
	    dataArray[dataArrayIndex] = resource.fetch();
	    if(dataArray[dataArrayIndex] == null) {
		throw Constants.makeFault("Concatenation: a file is empty ");
            }
            dataArrayIndex++;
	}
        
	ArrayOfDouble[] da = dataArray[0].getArray();
	if(da == null) {
	    throw Constants.makeFault("Concatenation: a file is empty ");
        }
	int size = da.length;
	int xSize = (da[0].getValue()).length;
	int totalSize = xSize;
	for(int i = 1; i < eprs.length; i++) {
	    da = dataArray[i].getArray();
	    if(da == null) {
		throw Constants.makeFault("Concatenation: a file is empty ");
            }
	    if(da.length != size) {
		throw Constants.makeFault("Concatenating dissimilar files ");
            }
	    int s = (da[0].getValue()).length;
	    totalSize+=s;
	}

	double[] x = new double[totalSize];
	double[] y = new double[totalSize];
	double[] z = new double[totalSize];
	int counter = 0;
	for(int i = 0; i < eprs.length; i++) {
	    da = dataArray[i].getArray();
	    double[] xTmp = da[0].getValue();
	    if(xTmp == null) {
		throw Constants.makeFault("Concatenation: "+
					  " null entry for x array in a file");
            }
	    double[] yTmp = null;
	    double[] zTmp = null;
	    if(size > 1) {
		yTmp = da[1].getValue();
		if(yTmp == null) {
                    throw Constants.makeFault("Concatenation: null entry for y"
                                              + " array in a file");
                }
		if(xTmp.length != yTmp.length) {
                    throw Constants.makeFault("Concatenation: files don't "
                                              + "contain x,y equal length ");
                }
	    }
	    if(size > 2) {
		zTmp = da[2].getValue();
		if (zTmp == null) {
		    throw Constants.makeFault("Concatenation: null entry for z"
                                              + "array in a file");
                }
                if (xTmp.length != zTmp.length) {
                    throw Constants.makeFault("Concatenation: files don't "
                                              + "contain x,z equal length ");
                }
	    } else {
		throw Constants.makeFault("Concatenation: too many "+
					  " arrays in files ");
            }
            
	    for(int j=0;j<xTmp.length;j++){
		x[counter]=xTmp[j];
		if(size > 1) {
		    y[counter]=yTmp[j];
                }
		if(size > 2) {
		    z[counter]=zTmp[j];
                }
		counter++;
	    }
	}

        if (size == 1){
            outResource.store(new Data(new ArrayOfDouble[] 
		{new ArrayOfDouble(x)}));
        } else if (size == 2){
            outResource.store(new Data(new ArrayOfDouble[] 
		{new ArrayOfDouble(x), new ArrayOfDouble(y)}));
        } else if (size == 3){
            outResource.store(new Data(new ArrayOfDouble[] 
		{new ArrayOfDouble(x),new ArrayOfDouble(y),
		 new ArrayOfDouble(z)}));
        } 
        return new VoidType();
    }

    private Resource getResource(EndpointReferenceType epr) 
        throws Exception {

        ResourceHome home = getServiceHome();
        QName keyName = home.getKeyTypeName();

        MessageElement reqElem = null;
        MessageElement[] elems = epr.getProperties().get_any();

        for (int i=0; i<elems.length; i++) {

            if (elems[i].getLocalName().equals(keyName.getLocalPart()) &&
                elems[i].getNamespaceURI().equals(keyName.getNamespaceURI())) {
                reqElem = elems[i];
                break;
            }
        }
        // FIXME check for null 
        String resourceId  = 
            (String)ObjectDeserializer.toObject(reqElem, 
                                                home.getKeyTypeClass());
        
        Resource resource = home.find(new SimpleResourceKey(keyName,
                                                            resourceId));
        return resource;
    }

    protected ResourceHome getServiceHome() throws Exception {
        
        ResourceContext context = ResourceContext.getResourceContext();
        return context.getResourceHome();
    }
}
