/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.filestore.service;

import org.globus.tutorial.impl.filestore.Constants;

import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.utils.AddressingUtils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.tutorial.types.filestore.Request;
import org.globus.tutorial.types.filestore.FileStoreException;

import org.apache.axis.components.uuid.UUIDGen;
import org.apache.axis.components.uuid.UUIDGenFactory;

import java.util.HashMap;

import org.apache.axis.message.addressing.EndpointReferenceType;

public class FileServiceHome extends ResourceHomeImpl {

    static Log logger = 
        LogFactory.getLog(FileServiceHome.class.getName());

    public static UUIDGen uuidGen = UUIDGenFactory.getUUIDGen();
    protected String fileStoreRoot = null;
    
    public void setFileStoreRoot(String fileStoreRoot_) {
        this.fileStoreRoot = fileStoreRoot_;
    }

    public String getFileStoreRoot() {
        return this.fileStoreRoot;
    }

    public ResourceKey create(Request request, String factoryResourceId)
        throws ResourceException {
        
        logger.debug("creating new resource");
        FileServiceResource resource = createResource(request, 
                                                      factoryResourceId);

         SimpleResourceKey key = 
             new SimpleResourceKey(keyTypeName, resource.getID());

        add(key, resource);
        logger.debug("Key for resource: " + key);
        return key;
    }

    protected FileServiceResource createResource(Request request, 
                                                 String factoryResourceId)
        throws ResourceException {
        
        return new FileServiceResource(request, uuidGen.nextUUID(), 
                                       factoryResourceId, this.fileStoreRoot);
    }
}
