/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.filestore.service;

import javax.xml.namespace.QName;

import school.tutorial.writer.XMLWriteFile;
import school.tutorial.writer.XMLWriteFileException;
import school.tutorial.parser.XMLParser;
import school.tutorial.parser.XMLParser1D;
import school.tutorial.parser.XMLParser2D;
import school.tutorial.parser.XMLParser3D;

import org.globus.tutorial.impl.filestore.Constants;

import org.globus.tutorial.impl.filestore.factory.FileStoreFactoryResource;
import org.globus.tutorial.impl.filestore.factory.FileStoreFactoryHome;

import org.globus.security.gridmap.GridMap;

import javax.naming.InitialContext;

import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.RemoveCallback;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceIdentifier;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceLifetime;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.wsrf.PersistenceCallback;
import org.globus.wsrf.impl.SimpleResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.wsrf.InvalidResourceKeyException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.axis.components.uuid.UUIDGen;
import org.apache.axis.components.uuid.UUIDGenFactory;

import org.globus.tutorial.types.filestore.Point;
import org.globus.tutorial.types.filestore.Triple;
import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.Request;
import org.globus.tutorial.types.filestore.FileStoreException;
import org.globus.tutorial.types.filestore.RequestFileType;
import org.globus.tutorial.types.filestore.ArrayOfDouble;

import java.util.Calendar;
import java.util.Vector;
import java.util.List;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;

import org.globus.wsrf.utils.XmlUtils;

import org.w3c.dom.Document;

import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;

import org.globus.wsrf.utils.FilePersistenceHelper;

import org.globus.util.Util;

public class FileServiceResource implements Resource, ResourceIdentifier, 
                                            ResourceLifetime, RemoveCallback,
                                            ResourceProperties {


    static Log logger = 
        LogFactory.getLog(FileServiceResource.class.getName());

    public static String CREATION_TIME_RP = "creationTime";
    public static String FILE_TYPE_RP = "fileType";
    public static String SIZE_RP = "size";
    public static String BYTES_RP = "bytes";
    
    ResourcePropertySet propSet;
    QName RP_SET = new QName(Constants.NS, "FileRPSet");

    protected String factoryId = null;
    protected String resourceId;
    protected Calendar terminationTime = null;
    protected RequestFileType fileType = null;
    protected Long bytes = null;
    protected Integer size = null;
    protected Calendar creationTime = null;
    protected String fileStoreRoot = null;

    public FileServiceResource() {
    }

    public FileServiceResource(Request request, String id, String factoryId,
                               String fileStoreRoot_) 
        throws ResourceException {

        this.resourceId = id;
        this.creationTime = Calendar.getInstance();
        Calendar now = this.creationTime;
        now.add(Calendar.MINUTE, request.getLifetimeInMins());
        this.terminationTime = now;
        this.fileType = request.getFileType();
        this.bytes = new Long(request.getFileSizeInBytes());
        this.factoryId = factoryId;
        this.fileStoreRoot = fileStoreRoot_;
        logger.debug("Created new file service resource");
        setUpDirectory();
        setResourceProperties();
    }

    protected void setUpDirectory() throws ResourceException {
        
        // check root 
        File file = new File(this.fileStoreRoot);
        if ((!file.exists()) || (!file.isDirectory())) {
            throw new ResourceException("Directory " + this.fileStoreRoot + 
                                        " does not exist");
        }
        // Create factory id  directory
        File callerDir = new File(this.fileStoreRoot, this.factoryId);
        callerDir.mkdirs();
    }

    // Resource properties
    public ResourcePropertySet getResourcePropertySet() {
        return this.propSet;
    }

    protected void setResourceProperties() {

        this.propSet = new SimpleResourcePropertySet(RP_SET);
        QName qName = new QName(Constants.NS, CREATION_TIME_RP);
        ResourceProperty prop = new SimpleResourceProperty(qName);
        prop.add(this.creationTime);
        this.propSet.add(prop);
        qName = new QName(Constants.NS, FILE_TYPE_RP);
        prop = new SimpleResourceProperty(qName);
        prop.add(this.fileType.getValue());
        this.propSet.add(prop);
        setSizeRP();
        setBytesRP();
    }
    
    private void setSizeRP() {
        QName qName = new QName(Constants.NS, SIZE_RP);
        ResourceProperty prop = new SimpleResourceProperty(qName);
        prop.add(this.size);
        this.propSet.add(prop);
    }

    private void setBytesRP() {
        QName qName = new QName(Constants.NS, BYTES_RP);
        ResourceProperty prop = new SimpleResourceProperty(qName);
        prop.add(this.bytes);
        this.propSet.add(prop);
    }

    public RequestFileType getFileType(){
	return this.fileType;
    }

    // ResourceIdentifier interface
    public Object getID() {
        return this.resourceId;
    }
    
    // Resource lifetime interface
    public void setTerminationTime(Calendar _terminationTime) {
        this.terminationTime   = _terminationTime;
    }

    public Calendar getTerminationTime() {
        return this.terminationTime;
    }

    public Calendar getCurrentTime() {
        return Calendar.getInstance();
    }

    // RemoveCallback
    public void remove() throws ResourceException {
	delete();
    }

    public void store(Data parameters) throws FileStoreException {
        
        if (fileType == RequestFileType.value1) {
	    this.store1D(parameters);
	} else if (fileType == RequestFileType.value2) {
	    this.store2D(parameters);
	} else {
	    this.store3D(parameters);
	}
    }

    private void store1D(Data parameters) throws FileStoreException {
        
        // Ideally error codes should be used.
        if (fileType != RequestFileType.value1) {
            throw Constants.makeFault("This operation can only be used with"
                                         + " 1D files");
        }
        ArrayOfDouble[] array = parameters.getArray();
	//do I need to check array contains something?
        if (array == null || array.length != 1 || array[0] == null) {
            throw Constants.makeFault("No data or wrong data has been passed");
        }
	double[] v = array[0].getValue();
        if (v == null ) {
            throw Constants.makeFault("No or wrong data has been passed");
        }
        int size1 = v.length;
        this.size = new Integer(size1);
        setSizeRP();
        Double[] values = new Double[size1];
	for(int i=0; i< size1;i++) {
	    values[i] = new Double(v[i]);
        }
        doStore(values);
    }

    private void store2D(Data parameters) throws FileStoreException {
        
        // Ideally error codes should be used.
        if (fileType != RequestFileType.value2) {
            throw Constants.makeFault("This operation can only be used with"
                                         + " 2D files");
        }
        ArrayOfDouble[] array = parameters.getArray();
        if (array == null || array.length !=  2 || 
	    array[0] == null || array[1] == null) {
            throw Constants.makeFault("No or wrong data has been passed");
        }
	double[] x = array[0].getValue();
	double[] y = array[1].getValue();
        if (x == null || y == null ) {
            throw Constants.makeFault("No or wrong data has been passed");
        }
        if (x.length != y.length) {
            throw Constants.makeFault("x and y data different length");
        }
        Point[] points = new Point[x.length];
	for(int i=0;i<points.length;i++) {
	    points[i]= new Point(x[i],y[i]);
	}
        int size1 = points.length;
        this.size = new Integer(size1);
        setSizeRP();
	doStore(points);
    }

    private void store3D(Data parameters) throws FileStoreException { 

       // Ideally error codes should be used.
        if (fileType != RequestFileType.value3) {
            throw Constants.makeFault("This operation can only be used with"
                                      + " 1D files");
        }
        
        ArrayOfDouble[] array = parameters.getArray();
        if (array == null || array.length != 3 || 
	    array[0] == null || array[1] == null || array[2]==null) {
            throw Constants.makeFault("No or wrong data has been passed");
        }
	double[] x = array[0].getValue();
	double[] y = array[1].getValue();
	double[] z = array[2].getValue();

        if (x == null || y == null || z == null) {
            throw Constants.makeFault("No or wrong data has been passed");
        }

        if (x.length != y.length || x.length != z.length) {
            throw Constants.makeFault("x, y, z data different length");
        }

        Triple[] triples = new Triple[x.length];
	for(int i=0;i<triples.length;i++) {
	    triples[i]= new Triple(z[i], new Point(x[i],y[i]));
	}
        int size1 = triples.length;
        this.size = new Integer(size1);
        setSizeRP();
        doStore(triples);
    }

    public void delete() throws ResourceException {

	String error = "FileStore: Delete: ";

        try{
	    File f = getFile();
	    String name = f.getPath();

            //	    if(!f.exists()) {
            //		throw new IOException("FileStore: Delete: File "+
            //                                         name+" not found");
            //            }
            
            if (f.exists()) {
                if(!f.delete()) {
                    throw new IOException("FileStore: Delete: Deletion"+
                                          " failed for "+name);
                }
            }
	}
        catch(IOException ioe){
            error += ioe.getMessage();
            throw new ResourceException(error);
        }

        // FIXME
        //        addToQuota();
        this.bytes = new Long(0);
    }


    public Data fetch() throws FileStoreException {
        
	Vector vec = doFetch();        
        if (vec == null) {
            // contents might have been removed
            return null;
        }
        int size = vec.size();
	if(size == 0) {
	    throw Constants.makeFault("No data fetched");
        }
        if (fileType == RequestFileType.value1) {
	    if(vec.get(0) instanceof Double){
		double[] values = new double[size];
		for(int i=0;i<size;i++) {
		    values[i]=((Double)vec.get(i)).doubleValue();
		}
		return new Data(new ArrayOfDouble[] 
		    {new ArrayOfDouble(values)});
	    } else {
		throw Constants.makeFault("Data is not 1D");
            }
        }
        if (fileType == RequestFileType.value2) {
	    if(vec.get(0) instanceof Point){
		Point[] points = new Point[size];
		vec.toArray(points);
		double[] x = new double[size];
		double[] y = new double[size];
		for(int i=0;i<size;i++){
		    x[i]=points[i].getX();
		    y[i]=points[i].getY();
		}
		return new Data(new ArrayOfDouble[] {new ArrayOfDouble(x),
						     new ArrayOfDouble(y)});
	    } else {
		throw Constants.makeFault("Data is not 2D");
            }
        }
        if (fileType == RequestFileType.value3) {
	    if(vec.get(0) instanceof Triple){
		Triple[] triples = new Triple[size];
		vec.toArray(triples);
		double[] x = new double[size];
		double[] y = new double[size];
		double[] z = new double[size];
		for(int i=0;i<size;i++){
		    Point p = triples[i].getP();
		    x[i]=p.getX();
		    y[i]=p.getY();
		    z[i]=triples[i].getH();
		}
		return new Data(new ArrayOfDouble[] {new ArrayOfDouble(x),
						     new ArrayOfDouble(y),
						     new ArrayOfDouble(z)});
	    } else {
		throw Constants.makeFault("Data is not 3D");
            }
        }

	return null;
    }

    private void doStore(Object[] data) throws FileStoreException{

        File resFile = getFile();

        logger.debug("Path in store" + resFile.getPath());
        //        if (resFile.exists()) {
            //	    throw Constants.makeFault("Problem with FileStore: "+
            //                                         "file already exists");
        //        }

	try{        
            XMLWriteFile w = new XMLWriteFile();
            Document doc = w.getDocument(data);
            if (doc == null) {
                return;
            }
            // add one for \n
            int docLength = XmlUtils.toString(doc).getBytes().length + 1;
            long storeLength = new Integer(docLength).longValue();
            long assignedBytes = this.bytes.longValue();
            logger.debug("Doc length " + storeLength);

            if (storeLength != assignedBytes) {
                //not the requested size, check and adjust quota
                long extraBytes = storeLength - assignedBytes;
                // FIXME
                //                checkQuota(extraBytes, storeLength);
                this.bytes = new Long(storeLength);
            }
            setBytesRP();
	    w.write(doc, resFile);
	}
	catch(XMLWriteFileException xwe){
	    throw Constants.makeFault("FileStore: Store: " + xwe.getMessage(), 
                                      xwe);
	}
    }

    private Vector doFetch() throws FileStoreException {

	String error = "FileStore: Fetch: ";
        File f = getFile();
        String name = f.getPath();
        try{
            if (!f.exists()) {
                ///		logger.warn("File: "+name+" not found");
                return null;
            }
            if (!f.isFile()) {
                //		throw new IOException("File: "+name+"
                //		is not a file");
                return null;
            }
            if (!f.canRead()) {
                //		throw new IOException("File: "+name+"
                //		cannot be read");
                return null;
            }
            if (f.length() == 0) {
                //		throw new IOException("File: "+name+"
                //		is empty");
                return null;
            }

            XMLParser p;
            if (fileType == RequestFileType.value1){
                p = new XMLParser1D();
            } else if(fileType == RequestFileType.value2){
                p = new XMLParser2D();
            } else {
                p = new XMLParser3D();
            }

	    Vector dataF = p.parse(f);
            if(dataF == null || dataF.size() == 0) {
                throw new IOException("Error with file: " + name 
                                      +" nothing read in");
            }
	    return dataF;
        } catch(IOException ioe) {
            error += ioe.getMessage();
            throw Constants.makeFault(error, ioe);
        } catch(SAXException se) {
            error += "Error in format of XML file "+name;
            throw Constants.makeFault(error, se);
        } catch(ParserConfigurationException pce) {
            error += "Error in configuring XML parser";
            throw Constants.makeFault(error, pce);
        }
    }

    private File getFile() {
        File tmp = new File(this.fileStoreRoot, this.factoryId);
        return new File(tmp + File.separator + "fileResource-" 
                        + this.resourceId);
    }

    protected String getPath() {
        return Constants.FACTORY_PATH;
    }

    protected FileStoreFactoryHome getFactoryHome() 
        throws FileStoreException {
        try {
            InitialContext context = new InitialContext();
            return (FileStoreFactoryHome)context
                .lookup(org.globus.wsrf.Constants.JNDI_SERVICES_BASE_NAME +
                        getPath() + 
                        org.globus.wsrf.Constants.HOME_NAME);
        } catch (Exception exp) {
            throw Constants.makeFault(exp.getMessage(), exp);
        }
    }

    protected FileStoreFactoryResource getFactoryResource()
        throws FileStoreException {

        FileStoreFactoryHome home = getFactoryHome();
        try {
            return (FileStoreFactoryResource)home.find(this.factoryId);
        } catch (ResourceException exp) {
            throw Constants.makeFault("Could not get factory resource"
                                         + exp.getMessage(), exp);
        }
    }

    protected void checkQuota(long diff, long storeLength) 
        throws FileStoreException {

        FileStoreFactoryResource resource = getFactoryResource();
        if (resource == null) {
            throw Constants.makeFault("Could not get factory resource"
                                         + " and ascertain quota");
        }

        boolean withinQuota = resource.checkQuota(diff);
        
        if (!withinQuota) {
            throw Constants.makeFault("Exceeded quota, cannot store." +
                                      "Requested bytes " + storeLength);
        }
    }

    protected void addToQuota() throws ResourceException {

        FileStoreFactoryResource resource = null;
        try {
            resource = getFactoryResource();
        } catch (FileStoreException exp) {
            throw new ResourceException("Error getting factory resource. "
                                        + exp.getMessage());
        }
        if (resource == null) {
            new ResourceException("Could not get factory resource to"
                                  + " update quota");
        }
        
        resource.addToQuota(this.bytes.longValue());
    }

}

