/*
 * XMLBuilder.java
 *
 * Created on 06 June 2005, 13:52
 */
package school.tutorial.builder;

import org.w3c.dom.Text;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.globus.tutorial.types.filestore.Point;
import org.globus.tutorial.types.filestore.Triple;

/**
 * Abstract Class for building a XML DOM document containing 1D, 2D or
 * 3D data.
 * @author sara
 */
public abstract class XMLBuilder {
    
    protected DocumentBuilder builder;
    protected Document doc;

    /** Creates a new instance of XMLBuilder */
    public XMLBuilder() throws ParserConfigurationException {
        DocumentBuilderFactory factory 
                    = DocumentBuilderFactory.newInstance();
        builder = factory.newDocumentBuilder();
    }

    /** Method for building a DOM document from an array of
     * Double, Point or Triple.
     * @param f holds the data
     * @return the DOM document produced 
     */
    public abstract Document build(Object[] f);

    /** Method to create an Element encoding the 
     * information for a Triple object
     * @param t the input Triple
     * @return the Element produced
     */
    protected Element createTriple(Triple t){
        Element e = doc.createElement("Triple");
        e.appendChild(createPoint(t.getP()));
        e.appendChild(createValue(t.getH(),"h"));
        return e;
    }
    /** Method to create an Element encoding the 
     * information for a Point object
     * @param p the input Point
     * @return the Element produced
     */
    protected Element createPoint(Point p){
        Element e = doc.createElement("p");
        e.appendChild(createValue(p.getX(),"x"));
        e.appendChild(createValue(p.getY(),"y"));
        return e;
    }
    /** Method to create an Element encoding the 
     * information for a double
     * @param x the input real value
     * @param name the tag name for the Element
     * @return the Element produced
     */
    protected Element createValue(double x, String name){
        Element e = doc.createElement(name);
        Text valueText = doc.createTextNode(""+x);
        e.appendChild(valueText);
        return e;
    }
    
    
}
