/*
 * Parser.java
 *
 * Created on 06 June 2005, 10:29
 */
package school.tutorial.parser;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Text;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import java.util.Vector;

import java.io.IOException;

import org.xml.sax.SAXException;

import org.globus.tutorial.types.filestore.Point;
import org.globus.tutorial.types.filestore.Triple;
/**
 * Class for reading in an XML file containing 1D, 2D or 3D data and 
 * and putting the data in a Vector of Double, Point or Triple.
 *
 * @author sara
 */
public abstract class XMLParser {

    private DocumentBuilder builder;

    /** Creates a new instance of Parser */
    public XMLParser() throws ParserConfigurationException{
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        builder = factory.newDocumentBuilder();
    }

    /** Method for reading an XML file and outputing the data
     * as a Vector.
     * @param f the input file.
     * @return the data extracted.
     */
    public Vector parse(java.io.File f) throws IOException, SAXException{
        Document doc = builder.parse(f);
        Element root = doc.getDocumentElement();
        return getContents(root);
    }

    /** Method for extracting the data from the document root element
     * @param e the document root element
     * @return the data extracted
     */
    protected abstract Vector getContents(Element e);

    /** Method for extracting a Triple (x,y,h)
    * @param e an Element in the document
    * @return the Triple extracted from the Element
    */
    protected Triple getTriple(Element e){
        if(e.hasChildNodes()){
            if(e.getFirstChild() instanceof Element &&
                    e.getLastChild() instanceof Element) {
                Element pointE = (Element) e.getFirstChild();
                Element hE = (Element) e.getLastChild();
                Point p = getPoint(pointE);
                double h = (getValue(hE)).doubleValue();
                return new Triple(h, p);
            }
        }
    return null;
    }
    /** Method for extracting a Point (x,y)
    * @param e an Element in the document
    * @return the Point extracted from the Element
    */
    protected Point getPoint(Element e){
        if(e.hasChildNodes()){
            Node xnode = e.getFirstChild();
            Node ynode = e.getLastChild();
            try{
		Node childx = xnode.getFirstChild();
                Text xText = (Text) childx;
                Text yText = (Text) ynode.getFirstChild();
                double x = Double.parseDouble(xText.getNodeValue().trim());
                double y = Double.parseDouble(yText.getNodeValue().trim());
                return new Point(x,y);
            }
            catch(NumberFormatException nfe){
            }
        }
        return null;
    }
    /** Method for extracting a Double
    * @param e an Element in the document
    * @return the real number extracted from the Element
    */
    protected Double getValue(Element e){
        if(e.hasChildNodes()){
            Text t = (Text) e.getFirstChild();
            String rnumS = t.getNodeValue().trim();
	    Double val;
            try{
                return new Double(Double.parseDouble(rnumS));
            } 
            catch(NumberFormatException nfe){
            }
        }

        return null;
    }
}
