/*
 * XMLWriteFile.java
 *
 * Created on 06 June 2005, 13:41
 */
package school.tutorial.writer;

import org.w3c.dom.Document;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import javax.xml.parsers.ParserConfigurationException;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;

import school.tutorial.builder.XMLBuilder;
import school.tutorial.builder.XMLBuilder1D;
import school.tutorial.builder.XMLBuilder2D;
import school.tutorial.builder.XMLBuilder3D;

import javax.xml.transform.stream.StreamResult;

import javax.xml.transform.dom.DOMSource;

import org.globus.tutorial.types.filestore.Point;
/**
 * Class for writing an array of Double, Point or Triple to a XML
 * file.
 *
 * @author sara
 */
public class XMLWriteFile {
    
    /** Creates a new instance of XMLWriteFile */
    public XMLWriteFile() {
    }

    /**
     * Generates a XML document from the data
     * @param data 
     * @return the XML document
     */
    public Document getDocument(Object[] data) throws XMLWriteFileException {

	String error = "";
        try{
            XMLBuilder builder;
            Document doc;
	    if(data.length == 0) return null;
            if(data[0] instanceof Double){
                builder = new XMLBuilder1D();
            } else if(data[0] instanceof Point){
                builder = new XMLBuilder2D();
            } else {
                builder = new XMLBuilder3D();
            }
            return builder.build(data);
        }  catch(ParserConfigurationException pce){
            error = "Error in configuring XML parser in XMLWriter";
        }          
	throw new XMLWriteFileException(error);
    }

    /** Method to write an array of Double, Point or Triple objects to
     * a XML file.
     * @param doc contains the data to be written
     * @param f the output file
     */
    public void write(Document doc, java.io.File f) 
	throws XMLWriteFileException {
	String error = "";
        try{
            Transformer t = TransformerFactory.newInstance().newTransformer();
            t.transform(new DOMSource(doc), new StreamResult(f));
	    PrintWriter pw = new PrintWriter(new FileWriter(f,true));
	    pw.println("\n");
	    pw.close();
	    return;
        }
	catch(IOException ioe){
	    error += "Error trying to write output";

	}
        catch(TransformerConfigurationException tce){
            error += "Error setting up transformer config in XMLWriter";
        }
        catch(TransformerException te){
            error += "Transformer exception in XMLWriter " + te.getMessage();
        }
	System.out.println(error);
	throw new XMLWriteFileException(error);
    }
}
