/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.randgen.service;

public class Box {

    //the corners of the bounding box
    private double lowerCornerX;
    private double lowerCornerY;
    private double upperCornerX;
    private double upperCornerY;

    public Box(double lcx, double lcy, double ucx, double ucy) 
        throws BoxException {

	if(lcx >= ucx || lcy >= ucy) {
            throw new BoxException("Box not defined" +
                                   " correctly: lower corner "+lcx+ " "+lcy
                                   +" upper corner "+ucx+" "+ucy);
        }
        this.lowerCornerX = lcx;
        this.lowerCornerY = lcy;
        this.upperCornerX = ucx;
        this.upperCornerY = ucy;
    }

    public Box(org.globus.tutorial.types.randgen.Box box) 
        throws BoxException {

        this.lowerCornerX = box.getLowerCornerX();
        this.lowerCornerY = box.getLowerCornerY();
        this.upperCornerX = box.getUpperCornerX();
        this.upperCornerY = box.getUpperCornerY();

	if(lowerCornerX >= upperCornerX || lowerCornerX >= upperCornerY) {
            throw new BoxException("Box not defined" +
                                   " correctly: lower corner "+lowerCornerX 
                                   + " "+ lowerCornerY
                                   +" upper corner "+ upperCornerX+" "
                                   +upperCornerY);
        }
    }

    public boolean check() {
	if (lowerCornerX >= upperCornerX || lowerCornerY >= upperCornerY) {
            return false;
        }
	return true;
    }

    /** Tests whether a 2D point is within the square
     * defined by this instance of Box.
     * @param x the x coordinate of the point to be tested
     * @param y the y coordinate of the point to be tested
     * @return true if the point is within the square
     */
    public boolean inBox(double x, double y) {
        return (x >= lowerCornerX) && (x <= upperCornerX) &&
	    (y >= lowerCornerY) && (y <= upperCornerY);
    }

    /** Returns the 2D point specifying the lower left hand
     * corner of the square
     * @return the lower left hand corner
     */
    public double getLowerCornerX() {
        return lowerCornerX;
    }

    /** Returns the 2D point specifying the lower left hand
     * corner of the square
     * @return the lower left hand corner
     */
    public double getLowerCornerY() {
        return lowerCornerY;
    }

    /** Returns the 2D point specifying the lower left hand
     * corner of the square
     * @return the lower left hand corner
     */
    public double getUpperCornerX() {
        return upperCornerX;
    }

    /** Returns the 2D point specifying the lower left hand
     * corner of the square
     * @return the lower left hand corner
     */
    public double getUpperCornerY() {
        return upperCornerY;
    }

    public void setUpperCornerY(double y) {
        upperCornerY = y;
    }

    public void setUpperCornerX(double x) {
        upperCornerX = x;
    }

    public void setLowerCornerY(double y) {
        lowerCornerY = y;
    }

    public void setLowerCornerX(double x) {
        lowerCornerX = x;
    }
 }
