/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.seqgen.service;

import java.rmi.RemoteException;
import javax.xml.rpc.Stub;


import org.globus.tutorial.types.seqgen.SeqGenReq;
import org.globus.tutorial.types.seqgen.SeqGenResponse;
import org.globus.tutorial.types.seqgen.SeqGeneratorException;

import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.ArrayOfDouble;

import org.globus.tutorial.filestore.FileService;
import org.globus.tutorial.filestore.service.FileServiceAddressingLocator;
import org.globus.tutorial.types.filestore.FileStoreException;

import javax.xml.rpc.ServiceException;

import org.globus.wsrf.utils.FaultHelper;

import java.util.Vector;

public class SeqGen {
    
    /** Generates a sequence of equally spaced real numbers
     */
    public SeqGenResponse generate(SeqGenReq req) 
        throws SeqGeneratorException, ServiceException {
        double start = req.getStart();
        double step = req.getStep();
        double limit = req.getLimit();
        EndpointReferenceType f = req.getURLFileName1D();

	if((start > limit)||(step <= 0)) {
	    System.out.println("Error with limits");
	   throw new SeqGeneratorException("Seq Gen: problem with sequence "
                                           + "variables: start=" + start 
                                           + " limit=" + limit + " step=" 
                                           + step + "\n rules: start <= limit,"
                                           + " step > 0");
        }

	String error = "";
        try {
            Data out = getSequence(start, step, limit);
            FileServiceAddressingLocator loc = 
                new FileServiceAddressingLocator();
            FileService service = loc.getFileServicePort(f);
            service.store(out);
            return new SeqGenResponse();
            
        } catch(FileStoreException fse){
            FaultHelper fh = new FaultHelper(fse);
            String er = fh.getDescriptionAsString();
            if (er.equals("This operation can only be used with 1D files")) {
                error = "Seq Gen: Input file should be a 1D file";
            } else {
                error= "Seq Gen: " + er;
            }
        } catch(RemoteException re){
	    error="Seq Gen: Remote error access FileStore :"+re.getMessage();
        }
	throw new SeqGeneratorException(error);
    }
    
    public Data getSequence(double start, double step, double limit) {
        
        //        int count = (int)Math.floor((limit-start)/step)+1;
        Vector values = new Vector();
	int i=0;
        for(double seq = start; seq <= limit; seq+=step) {
            values.add(new Double(seq));
            i++;
        }

        double[] send = new double[i];
        for(int j = 0; j < i; j++){
            send[j]=((Double)values.get(j)).doubleValue();
        }
        
        Data out = new Data(new ArrayOfDouble[] { 
            new ArrayOfDouble(send)});
        return out;
    }
}
