/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.surface.service;

import javax.xml.namespace.QName;

import org.globus.tutorial.types.surface.ProbeException;

import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.ArrayOfDouble;

import org.globus.wsrf.Resource;

import org.globus.tutorial.impl.randgen.service.Box;
import org.globus.tutorial.impl.randgen.service.BoxException;

import java.util.Properties;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.BufferedReader;

import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;

import org.globus.wsrf.impl.SimpleResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertySet;

import org.globus.tutorial.surface.SurfaceResource;

public class PillarResource extends SurfaceResource {

    public final static String PILLAR_DATA_DIR = "pillarDataDir";
    public final static String NO_OF_PILLARS = "numberOfPillars";
    public final static String PILLAR_DATA_FILE = "pillarDataFile";
    public final static String PILLAR_DATA_TXT_FILE = "pillarDataTxtFile";
    public final static String BOX_DATA_FILE = "boxDataFile";

    private final static byte[] temp1 = {97, 110, 102, 114, 117, 116, 110, 99, 115, 57, 50, 56, 55, 101, 106, 107, 100, 110, 102, 106, 102, 104, 119, 119, 110, 102, 115, 104, 101, 104};

    String boxDataFile = null;
    String pillarsDataFile = null;
    String pillarsDataTxtFile = null;
    String pillarDataDir = null;
    int numOfPillars = 0;
    private Pillar[] pillars = null;

    protected Data probe(Data input) throws ProbeException {

	Data output = null;
	try{
	    output = genSurface(input);
	} catch(BoxException be){
            throw makeFault("Probe function error", be);
	}
        return output;
    }

    protected void setupConfig(Properties prop) throws ProbeException {

        String err = "Error in set up config";

        // get pillar values file.
        this.pillarDataDir = prop.getProperty(PILLAR_DATA_DIR);
        this.numOfPillars = 
            new Integer(prop.getProperty(NO_OF_PILLARS)).intValue();
        this.pillars = new Pillar[this.numOfPillars + 1];
        this.pillarsDataFile = prop.getProperty(PILLAR_DATA_FILE);
        this.pillarsDataTxtFile = prop.getProperty(PILLAR_DATA_TXT_FILE);
        this.boxDataFile = prop.getProperty(BOX_DATA_FILE);

	BufferedReader br = null;

	String line = null;
        try {
	    br = new BufferedReader(new FileReader(this.pillarDataDir
                                                   + File.separator + 
                                                   this.boxDataFile));
            line = br.readLine();
        } catch (FileNotFoundException exp) {
            throw makeFault(err + exp.getMessage(), exp);
        } catch (IOException exp) {
            throw makeFault(err + exp.getMessage(), exp);
        } finally {
            try {
                br.close();
            } catch (Exception e) {}
        }
        
	if(line != null && line.trim() != ""){
	    String[] values = line.split("\\s");
	    if(values.length != 4) {
		throw makeFault(err + "Not enough data in Box file");
            }
	    double xL = Double.parseDouble(values[0]);
	    double yL = Double.parseDouble(values[1]);
	    double xU = Double.parseDouble(values[2]);
	    double yU = Double.parseDouble(values[3]);
            try {
                this.bound = new Box(xL,yL,xU,yU);
            } catch (BoxException exp){
                throw makeFault(err, exp);
            }
	} else {
	    throw makeFault(err + "Not enough data in Box file");
        }

        try {
            //read in  data
            readInData();
        } catch (IOException exp) {
            throw makeFault(err + exp.getMessage(), exp);
        } catch (EncryptionException exp) {
            throw makeFault(err + exp.getMessage(), exp);
        }
    }

    private static String bytes2String( byte[] bytes )
    {
	StringBuffer stringBuffer = new StringBuffer();
	for (int i = 0; i < bytes.length; i++)
	    {
		stringBuffer.append( (char) bytes[i] );
	    }
	return stringBuffer.toString();
    }

    private void readInData() throws IOException,
                                     NumberFormatException,
                                     EncryptionException {

        String filename = this.pillarDataDir + File.separator 
            + this.pillarsDataFile;
	File f = new File(filename);
	if(!f.exists()) {
	    throw new IOException(filename + " not found");
        }

	StringEncrypter se = new StringEncrypter(bytes2String(temp1));
	BufferedReader br = new BufferedReader(new FileReader(f));
	for(int i=1; i <= this.numOfPillars;i++){
	    String line = se.decrypt(br.readLine());
	    String[] values = line.split("\\s");
	    if(values.length != 12) {
		throw new IOException("Not enough data in "+
				      filename +  "file: " + line);
            }
	    Pillar p = new Pillar();
	    p.height = Double.parseDouble(values[0]);
	    p.x_width = Double.parseDouble(values[1]);
	    p.y_width = Double.parseDouble(values[2]);
	    p.blh_x = Double.parseDouble(values[3]);
	    p.blh_y = Double.parseDouble(values[4]);
	    p.pixel_step = Double.parseDouble(values[5]);
	    p.offset_x = Double.parseDouble(values[6]);
	    p.offset_y = Double.parseDouble(values[7]);
	    p.x_dim = Double.parseDouble(values[8]);
	    p.y_dim = Double.parseDouble(values[9]);
	    p.z_dim = Double.parseDouble(values[10]);
	    p.pixel_height = Double.parseDouble(values[11]);
            double b1 = background( p.blh_x, p.blh_y );
            double b2 = background( p.blh_x + p.x_width, p.blh_y );
            double b3 = background( p.blh_x, p.blh_y + p.y_width );
            double b4 = background( p.blh_x + p.x_width, p.blh_y + p.y_width );
            p.baseHeight = Math.max(Math.max(b1,b2),Math.max(b3,b4));
	    pillars[i]=p;
            /* FIXME
            System.out.println("Pillar data is ");
            System.out.println(p.height + " " + p.x_width + " " + p.y_width 
                               + " " + p.blh_x + " " + p.blh_y + " "
                               + p.pixel_step + " " + p.offset_x + " "
                               + p.offset_y + " " + p.x_dim + " " + p.y_dim
                               + " " + p.z_dim + " " + p.pixel_height + " " 
                               + p.baseHeight);
            */
	}
	br.close();
        String filename2 = this.pillarDataDir + File.separator 
            + this.pillarsDataTxtFile;
	f = new File(filename2);
	if(!f.exists()) {
	    throw new IOException(filename2 + " not found");
        }
	br = new BufferedReader(new FileReader(f));

	for(int i=1; i <= this.numOfPillars;i++){
	    pillars[i].setPixels(br, se);
	}
	br.close();
    }
    
    private double background(double x, double y){
	final double epsilon = 0.0001;
	final double a = 0.256;
	final double k = 5.0;
	final double t = 0.2;
	double r = Math.sqrt(x*x+y*y);
	return r*r*a*Math.exp(k/(r+epsilon))*Math.sin(t*r*Math.PI);
    }

    /** Generates the surface samples
     * @param bound the bounding box within which the surface is defined
     */
    public Data genSurface(Data input) throws BoxException, ProbeException{

	ArrayOfDouble[] arrays = input.getArray();
	if (arrays.length != 2) {
	    throw makeFault("need x and y, not " + arrays.length +" arrays");
        }

	double[] x = arrays[0].getValue();
	double[] y = arrays[1].getValue();

	if (x.length != y.length) {
	    throw makeFault(" not the same number of x and y ");
        }

	if (x.length > this.maxSampleSize) {
	    throw makeFault("maximum number of points: "+
                            this.maxSampleSize +" exceeded");
	}

	double[] h = new double[y.length];
	for(int i=0; i < y.length;i++){
	    if (!this.bound.inBox(x[i],y[i])) {
                throw new BoxException("Point out of bounds "+x[i]+" "+y[i]);
            }
            h[i] = surfaceFunction(x[i],y[i]);
            //            System.out.println("Surface function: " + x[i] + " " + y[i] +
            //                               " " + h[i]);
        }

        return new Data(new ArrayOfDouble[] {arrays[0], arrays[1],
					     new ArrayOfDouble(h)});
    }

    /** The surface function which returns the height of
     * a point on the surface for given 2D coordinates
     * @param x  the x coordinate
     * @param y  the y coordinate
     * @return the height of the surface
     */
    protected double surfaceFunction(double x, double y) {
        //	try{
        //	    Thread.sleep(1);
        //	} catch(InterruptedException ie){}

	for(int m=1; m <= this.numOfPillars; m++){
	    if(pillars[m].inside(x,y)){
		return pillars[m].pillarSurface(x,y);
	    }
	}
	return background(x,y);
    }
}
