/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.surface.service;

import javax.xml.namespace.QName;

import org.globus.tutorial.types.surface.ProbeException;

import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.ArrayOfDouble;

import org.globus.wsrf.Resource;

import org.globus.tutorial.impl.randgen.service.Box;
import org.globus.tutorial.impl.randgen.service.BoxException;

import java.util.Properties;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.BufferedReader;

import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;

import org.globus.wsrf.impl.SimpleResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertySet;

import org.globus.tutorial.surface.SurfaceResource;

import org.globus.tutorial.surface.Shape;

public class ShapesResource extends SurfaceResource {

    public final static String SHAPES_DATA_DIR = "shapesDataDir";

    public final static String BOX_DATA_FILE = "boxDataFile";
    String shapesDataDir = null;
    private Shape[] shapes = new Shape[300];
    private int count = 0;

    protected void setupConfig(Properties prop) throws ProbeException {
     
        String err = "Error in set up config";
   
        this.shapesDataDir = prop.getProperty(SHAPES_DATA_DIR);
        String boxDataFile = prop.getProperty(BOX_DATA_FILE);

        String line = null;
	BufferedReader br =  null;
        try {
	    br = new BufferedReader(new FileReader(shapesDataDir 
                                                   + File.separator 
                                                   + boxDataFile));
            
            line = br.readLine();
        } catch (FileNotFoundException exp) {
            throw makeFault(err + exp.getMessage(), exp);
        } catch (IOException exp) {
            throw makeFault(err + exp.getMessage(), exp);
        } finally {
            try {
                br.close();
            } catch (Exception e) {}
        }

        if (line != null && line.trim() != ""){
            String[] values = line.split("\\s");
            if (values.length != 4) {
                throw makeFault("Not enough data in Box file");
            }
            double xL = Double.parseDouble(values[0]);
            double yL = Double.parseDouble(values[1]);
            double xU = Double.parseDouble(values[2]);
            double yU = Double.parseDouble(values[3]);
            try {
                this.bound = new Box(xL,yL,xU,yU);
            } catch (BoxException exp){
                throw makeFault(err, exp);
            }
        } else {
            throw makeFault(err + "Not enough data in Box file");
        }

        try {
            //read in  data
            readInData();
        } catch (IOException exp) {
            throw makeFault(err + exp.getMessage(), exp);
        }
    }

    protected Data probe(Data input) throws ProbeException {

	Data output = null;
	try{
	    output = genSurface(input);
	} catch(BoxException be){
            throw makeFault("Probe function error", be);
	}
        return output;
    }

    private void readInData() throws IOException, NumberFormatException{
	java.io.File f = new java.io.File(this.shapesDataDir + 
                                          File.separator + "ConesData.txt");
	if(f.exists()){
	    BufferedReader br = 
		new BufferedReader(new FileReader(f));
	    String line = br.readLine();
	    while (line != null && line.trim() != ""){
		double[] x = extract(line,4,"Cones");
		shapes[count] = new Cone(x[0],x[1],x[2],x[3]);
		count++;
		line = br.readLine();
	    }
	    br.close();
	}
	java.io.File f2 = new java.io.File(this.shapesDataDir + 
                                           File.separator + "CuboidsData.txt");
	if(f2.exists()){
	    BufferedReader br = 
		new BufferedReader(new FileReader(f2));
	    String line = br.readLine();
	    while (line != null && line.trim() != ""){
		double[] x = extract(line,5,"Cuboids");
		shapes[count] = new Cuboid(x[0],x[1],x[2],x[3],x[4]);
		count++;
		line = br.readLine();
	    }
	    br.close();
	}

	f = new java.io.File(this.shapesDataDir + File.separator 
                             + "PyramidsData.txt");
	if(f.exists()){
	    BufferedReader br = 
		new BufferedReader(new FileReader(f));
	    String line = br.readLine();
	    while (line != null && line.trim() != ""){
		double[] x = extract(line,5,"Pyramids");
		shapes[count] = new Pyramid(x[0],x[1],x[2],x[3],x[4]);
		count++;
		line = br.readLine();
	    }
	    br.close();
	}
	f = new java.io.File(this.shapesDataDir + File.separator 
                             + "PipesData.txt");
	if(f.exists()){
	    BufferedReader br = 
		new BufferedReader(new FileReader(f));
	    String line = br.readLine();
	    while (line != null && line.trim() != ""){
		double[] x = extract(line,5,"Pipes");
		shapes[count] = new Pipe(x[0],x[1],x[2],x[3],x[4]);
		count++;
		line = br.readLine();
	    }
	    br.close();
	}
	f = new java.io.File(this.shapesDataDir + File.separator 
                             + "HemispheresData.txt");
	if(f.exists()){
	    BufferedReader br = 
		new BufferedReader(new FileReader(f));
	    String line = br.readLine();
	    while (line != null && line.trim() != ""){
		double[] x = extract(line,3,"Hemispheres");
		shapes[count] = new Hemisphere(x[0],x[1],x[2]);
		count++;
		line = br.readLine();
	    }
	    br.close();
	}
	f = new java.io.File(this.shapesDataDir + File.separator 
                             + "CylindersData.txt");
	if(f.exists()){
	    BufferedReader br = 
		new BufferedReader(new FileReader(f));
	    String line = br.readLine();
	    while (line != null && line.trim() != ""){
		double[] x = extract(line,4,"Cylinders");
		shapes[count] = new Cylinder(x[0],x[1],x[2],x[3]);
		count++;
		line = br.readLine();
	    }
	    br.close();
	}
    }

    private double[] extract(String line,int num, String fileID) 
	throws IOException, NumberFormatException{
	double[] x = new double[num];
	String[] values = line.split("\\s");
	if(values.length != num) 
	    throw new IOException("Not enough data in "+
				  fileID+  "file");
	for(int i=0; i< num;i++){
	    x[i] = Double.parseDouble(values[i]);
	}
	return x;
    }

    /** Generates the surface samples
     * @param bound the bounding box within which the surface is defined
     */
    public Data genSurface(Data input) throws BoxException, ProbeException{

	ArrayOfDouble[] arrays = input.getArray();
	if (arrays.length != 2) {
	    throw makeFault("need x and y, not " + arrays.length +" arrays");
        }

	double[] x = arrays[0].getValue();
	double[] y = arrays[1].getValue();

	if(x.length != y.length) {
	    throw makeFault("something wrong: x and y arrays are"+
                            " not the same length");
	}
	if(y.length > this.maxSampleSize) {
	    throw makeFault("maximum number of points: "+
                            this.maxSampleSize +" exceeded");
	}
	double[] h = new double[y.length];
	for(int i=0; i < y.length;i++){
	    if(!this.bound.inBox(x[i],y[i])) throw new BoxException(
                            "Point out of bounds "+x[i]+" "+y[i]);
            h[i] = surfaceFunction(x[i],y[i]);
        }

        return new Data(new ArrayOfDouble[] {arrays[0], arrays[1],
					     new ArrayOfDouble(h)});
    }

    /** The surface function which returns the height of
     * a point on the surface for given 2D coordinates
     * @param x  the x coordinate
     * @param y  the y coordinate
     * @return the height of the surface
     */
    protected double surfaceFunction(double x, double y) {

	final double plane  = 41.2;
	for(int m=0; m < count; m++){
	    if(shapes[m].inside(x,y)){
		return plane + shapes[m].elevation(x,y);
	    }
	}
	return plane;
    }
}
