/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.surface.service;

import java.rmi.RemoteException;
import javax.xml.rpc.Stub;

import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.tutorial.impl.randgen.service.Box;
import org.globus.tutorial.impl.randgen.service.BoxException;

import org.globus.tutorial.types.surface.ProbeException;

import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.ArrayOfDouble;
import org.globus.tutorial.types.filestore.VoidType;
import org.globus.tutorial.filestore.FileService;
import org.globus.tutorial.filestore.service.FileServiceAddressingLocator;
import org.globus.tutorial.types.filestore.FileStoreException;

import javax.xml.rpc.ServiceException;

import org.globus.wsrf.ResourceContext;

import org.globus.wsrf.utils.FaultHelper;

import org.globus.tutorial.types.surface.ProbeReq;

import org.globus.tutorial.surface.SurfaceResource;

/**
 * Class for sampling a 3D defined surface, where the samples are taken
 * at specified 2D coordinates.
 * @author sara
 */
public class ShapesSurface {


    /** Samples the surface
     * @param in the file containing the 2D sampling coordinates
     * @param out the file for outputting the 3D samples of the surface
     */
    public void probe(ProbeReq parameters) throws ProbeException, 
                                                  ServiceException {


	String error = "Probe: ";

        EndpointReferenceType in = parameters.getURLFileName2D();
        EndpointReferenceType out = parameters.getURLFileName3D();
        
        FileServiceAddressingLocator loc = 
            new FileServiceAddressingLocator();
        FileService service = loc.getFileServicePort(in);
        Data input = null;
        try {
            input = service.fetch(new VoidType());
        } catch (FileStoreException fse){
            FaultHelper fh = new FaultHelper(fse);
            String er = fh.getDescriptionAsString();
            if (er.equals("This operation can only be used with 2D "
                          + "files")) {
                throw SurfaceResource.makeFault("Probe: Input file should be a"
                                                + "2D file");
            } else {
                throw SurfaceResource.makeFault("Probe: " + er);
            }
        } catch(RemoteException re){
            throw SurfaceResource.makeFault("Probe: Remote error: " ,re);
	}

	ShapesResource resource = null;
	try {
	    resource =
		(ShapesResource)ResourceContext.getResourceContext()
		.getResource();          
	} catch(RemoteException re){
	    throw SurfaceResource.makeFault("Probe: remote error: ", re);
	}

	Data output = null;
	try {
	    output = resource.genSurface(input);
	} catch(BoxException be){
            throw SurfaceResource.makeFault("Probe function error: " , be);
	}

	try {
	    loc = new FileServiceAddressingLocator();
	    service = loc.getFileServicePort(out);
	    service.store(output);
	    return;
	} catch (FileStoreException fse){
	    FaultHelper fh = new FaultHelper(fse);
	    String er = fh.getDescriptionAsString();
	    if (er.equals("This operation can only be used with 3D "
			  + "files")) {
		throw SurfaceResource.makeFault("Probe: output file should be"
                                                + "a 3D file");
	    } else {
		throw SurfaceResource.makeFault("Probe: " + er);
	    }
	} catch(RemoteException re){
	    error += "Remote error: " + re.getMessage();
            throw SurfaceResource.makeFault(error, re);
	}
    }
}
