package org.globus.tutorial.impl.surface.service;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.File;
import java.io.IOException;
import java.io.FileWriter;
import java.io.PrintWriter;

import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.KeySpec;

import javax.crypto.Cipher;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import javax.crypto.spec.DESedeKeySpec;

import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

/** Class for encrypting strings.
 *
 */
public class StringEncrypter
{
	
    public static final String DESEDE_ENCRYPTION_SCHEME = "DESede";
    public static final String DES_ENCRYPTION_SCHEME = "DES";
    public static final String DEFAULT_ENCRYPTION_SCHEME = "DES";
	
    private KeySpec keySpec;
    private SecretKeyFactory	keyFactory;
    private Cipher	cipher;
	
    private static final String	UNICODE_FORMAT	= "UTF8";

    public StringEncrypter( String encryptionKey ) throws EncryptionException
    {
	this( DEFAULT_ENCRYPTION_SCHEME, encryptionKey  );
    }


    public StringEncrypter( String encryptionScheme, String encryptionKey )
	throws EncryptionException{

	if ( encryptionKey == null )
	    throw new IllegalArgumentException( "encryption key was null" );
	if ( encryptionKey.trim().length() < 24 )
	    throw new IllegalArgumentException("encryption key was less "+
					       "than 24 characters" );

	try{
	    byte[] keyAsBytes = encryptionKey.getBytes( UNICODE_FORMAT );
	    
	    if ( encryptionScheme.equals( DESEDE_ENCRYPTION_SCHEME) ){
		keySpec = new DESedeKeySpec( keyAsBytes );
	    }
	    else if ( encryptionScheme.equals( DES_ENCRYPTION_SCHEME ) ){
		keySpec = new DESKeySpec( keyAsBytes );
	    }
	    else{
		throw new IllegalArgumentException( "Encryption scheme not supported: "
						    + encryptionScheme );
	    }

	    keyFactory = SecretKeyFactory.getInstance( encryptionScheme );
	    cipher = Cipher.getInstance( encryptionScheme );

	}
	catch (InvalidKeyException e){
	    throw new EncryptionException( e.getMessage() );
	}
	catch (UnsupportedEncodingException e){
	    throw new EncryptionException( e.getMessage() );
	}
	catch (NoSuchAlgorithmException e){
	    throw new EncryptionException( e.getMessage() );
	}
	catch (NoSuchPaddingException e){
	    throw new EncryptionException( e.getMessage() );
	}

    }

    public String encrypt( String unencryptedString ) 
	throws EncryptionException{

	if ( unencryptedString == null || 
	     unencryptedString.trim().length() == 0 )
	    throw new IllegalArgumentException("unencrypted string was "+
					       "null or empty" );
	
	try{
	    SecretKey key = keyFactory.generateSecret( keySpec );
	    cipher.init( Cipher.ENCRYPT_MODE, key );
	    byte[] cleartext = unencryptedString.getBytes( UNICODE_FORMAT );
	    byte[] ciphertext = cipher.doFinal( cleartext );
		
	    BASE64Encoder base64encoder = new BASE64Encoder();
	    return base64encoder.encode( ciphertext );
	}
	catch (Exception e){
	    throw new EncryptionException( e.getMessage() );
	}
    }

    public String decrypt( String encryptedString ) throws EncryptionException
    {
	if ( encryptedString == null || encryptedString.trim().length() <= 0 )
	    throw new IllegalArgumentException( "encrypted string was null or empty" );
	
	try
	    {
		SecretKey key = keyFactory.generateSecret( keySpec );
		cipher.init( Cipher.DECRYPT_MODE, key );
		BASE64Decoder base64decoder = new BASE64Decoder();
		byte[] cleartext = base64decoder.decodeBuffer( encryptedString );
		byte[] ciphertext = cipher.doFinal( cleartext );
		
		return bytes2String( ciphertext );
	    }
	catch (Exception e)
	    {
		throw new EncryptionException( e.getMessage() );
	    }
    }
    
    private static String bytes2String( byte[] bytes )
    {
        StringBuffer stringBuffer = new StringBuffer();
        for (int i = 0; i < bytes.length; i++)
        {
            stringBuffer.append( (char) bytes[i] );
        }
        return stringBuffer.toString();
    }

    public static void main(String[] args) throws EncryptionException, IOException
    {
        /* Translate strings to bytes */
        /*
        for (int i = 0; i < args.length; i++)
        {
            System.out.println("String: " + args[i]);
            System.out.print("Bytes: {");
            String arg = args[i];
            for (int j = 0; j < arg.length(); j++)
            {
                System.out.print( (int) arg.charAt(j));
                if (j != arg.length() - 1)
                {
                    System.out.print(", ");
                }
            }
            System.out.println("}");
        }
        */
        /* Encrypt a file */
        String inname = args[0];
        String outname = inname + ".en";
        System.out.println("Input: " + inname);
        System.out.println("Output: " + outname);
        File infile = new File(inname);
        File outfile = new File(outname);
        StringEncrypter se = new StringEncrypter("anfrutncs9287ejkdnfjfhwwnfsheh");

        if(!outfile.createNewFile())
            throw new IOException("Probe: File: "+outname+
                                  " already exists");

        PrintWriter     pw = new PrintWriter(new FileWriter(outfile));
        BufferedReader  fr = new BufferedReader(new FileReader(infile));

        String line = fr.readLine();
        System.out.println("Read line: '" + line + "'");
        while (line != null) 
        {
            String en_line = se.encrypt(line);
            StringBuffer sb = new StringBuffer();
            for (int i=0; i < en_line.length(); i++)
            {
                if (en_line.charAt(i) != '\n') 
                    sb.append(en_line.charAt(i));
            }
            pw.println(sb.toString());
            System.out.println("Ecrypted version: '" + sb.toString() + "'");
            line = fr.readLine();
            System.out.println("Read line: '" + line + "'");
        } 
        fr.close();
        pw.close();

        return;
    }
    
}
